/* ATSP Solver using Simulated Annealing and the 3-opt algorithm */
/* Glenn Dicus, Bart Jaworski, and Joseph Ou-Yang */
/* (c) 1996 All rights reserved                  */




#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <assert.h>
#include <time.h>

	/*
	  stdlib.h - used to define the return values of malloc(), free()
			and exit().
	*/


#define BOOL            int
#define FALSE           0
#define TRUE            1
#define MAX_NAME_LEN    20
#define RAND_MAX        32767
#define MAX 		1000
#define ZERO            .00000000001

BOOL MST=FALSE;
BOOL Entered;

BOOL TRD_RAND;			//This global constant is true if user
				//wishes to create random matrix.

long double start,finish;  	//Variables used to establish CPU times.

long double TEMP=0;		//Annealing parameters.
long double INIT_TEMP=0;
long double ALPHA=0;

int * tour;

int * length = (int*) new int [sizeof(int)];

int ** matrix;			//matrix is the two dimensional cost matrix.

int inf;			//Variable holds the value of infinity
				//as specified in input file.
FILE *fp;			//File pointer to input file.

typedef struct edge		
{
  int c1;
  int c2;
} Edge_Type;





int * key;     			//Used for Prim's algorithm.
int * VISITED;
int * parent;


/* Strings used for user option processing */

char * process_MST = (char*) new char [sizeof(char)];
char * display_matrix = (char*) new char [sizeof(char)];
char * detailedMST = (char*) new char [sizeof(char)];
char * detailedTSP = (char*) new char [sizeof(char)];
char * file_name = (char*) new char [sizeof(char)];
char * which_matrix = (char*) new char [sizeof(char)];
char * which_opt = (char*) new char [sizeof(char)];
char * no_new = (char*) new char [sizeof(char)];
 
/* end string processing */
   

void copytour(int *, int *, int*, int);

int init_tour(int **,int *, int, int, int, int, int *);

int do3opt(int**, int*, int*, int);

void result(int *, int);

int mod_int(int, int);

long double mod_long_double(long double, long double);

int div(int, int);


/*Returns TRUE and minimum vertex u such that u has not been visited;
  vertex u is adjacent to a visited vertex p;and the edge between p and u 
  is minimum.*/	  
      

BOOL extract_min(int *VISITED, int **cost, int n, int *u, BOOL FIRST){

	int i,j;
	int min_cost;

	min_cost = inf;
	if(FIRST==TRUE){
	   *u = 0;
	   VISITED[0] = TRUE;
	   return TRUE;
	}
	else
	   for(i=0;i<n;i++)
	      if (VISITED[i]) 
	         for (j=0;j<n;j++) 
	      	    if((VISITED[j]==FALSE)&&(cost[i][j]<min_cost)&&(i!=j)){
		       *u = j;
		       min_cost = cost[i][j];
		    }
	if(min_cost!=inf){
	   VISITED[*u] = TRUE;
	   return TRUE;
	}
	else
	   return FALSE;
}
			   	   


//returns minimum spanning tree using Prim's algorithm.
//This implementation of Prim's algorithm can be optimized from O(V^3) to 
//O(E + VlgV) using a Fibonacci heap.

BOOL MST_prim(int **cost, int *parent, int n){

	int * u = (int*) new int [sizeof(int)];
	BOOL FIRST=TRUE;
	int its=0;
	int i,v;



	key = (int*) new int [n*sizeof(int)];
	VISITED   = (int*) new int [n*sizeof(int)];
	for (i=0;i<n;i++) {
	   key[i] = inf;
	   VISITED[i] = FALSE;
	   parent[i] = -1;
	}
	
	key[0] = 0;
	
	while (extract_min(VISITED,cost,n,u,FIRST)==TRUE){
	   its++;
	   FIRST = FALSE;
	   for(v=0;v<n;v++)
	      if((VISITED[v]==FALSE)&&(cost[*u][v] < key[v])&&(v!=*u)){
		 parent[v] = *u;
		 key[v] = cost[*u][v];	   
	      }
	}
	if (its == n)
	   return TRUE;
	else
	   return FALSE;

	
}


/* Print out the resultant data to a file */
void result(int *length, int n)
{
   FILE *file;
   char oper[6];
   long double SEC;
   int HOUR, MIN;

   if(TRD_RAND)
     oper = "3opt  ";
   else
     oper = "2+1opt";
   SEC = mod_long_double(finish-start,60.0);
   MIN = div((int)(finish-start),60);
   HOUR = div((int)(finish-start), 3600);
   if((file=fopen("results.dat","a"))==NULL)
      printf("The file is not available");
   else
      fprintf(file,"%6s   %6s   %d   %5.5lf   %5.5lf   %d:%d:%.0f  %6d \n",
	      file_name,oper, n, INIT_TEMP, ALPHA, HOUR,MIN, SEC, *length);
   fclose(file);
}


int mod_int(int x, int y){

	if ((x-y)>=0)
	   return mod_int(x-y,y);
	else
	   return x;
}

long double mod_long_double(long double x, long double y){

	if ((x-y)>=0)
	   return mod_long_double(x-y,y);
	else
	   return x;
}

int div(int x, int y){

	int tmp=0;

	while((x-y)>=0){
	  tmp++;
	  x-=y;
   	}
	return tmp;
}	   	   

/* Copy the tour */

void copytour(int *tour ,int *for_copy, int *rev_copy, int n ){
    int i;

    for (i=0; i<n; i++) {
        rev_copy[tour[i]] = i;
	for_copy[i] = tour[i];
      }
}

// Finds initial tour of the input data.
// The time complexity of this algorithm is T(n) = n*T(n-1) + n. 

int init_tour(int ** cost,int *visited, int depth, int n, int i, int j, int *length){


	fflush(stderr);
	int l=0;
	
	if (depth == (n-1))
	   if (cost[i][0]!=inf) {
		visited[i] = 0;
		*length += cost[i][0];
		return TRUE;
	   }
	   else
		return FALSE;
	else {
	   while (l++<n){
		if ((cost[i][j]!=inf)&&(visited[j]<0)&&(i!=j)) {
		   visited[i] = j;
		   *length += cost[i][j];
		   if(!init_tour(cost,visited,++depth, n,j,i,length)){
			visited[i] = -1;
			*length -= cost[i][j];
			--depth;
		   }
		   else
			return TRUE;
		}
		j = mod_int(++j,n);
	   }
	   return FALSE;
	}
}


/* returns gain if found a successfull 3opt move
   else it returns infinity */

/* Take out 3 random edges and replace them hopefully with 3 better ones */


int do3opt(int**cost, int *for_tour,int *rev_tour, int n) { 

    Edge_Type x1,x2,x3;
    Edge_Type y1,y2,y3;
    Edge_Type tmp_edge;
    int x1_iter = 0;
    int x2_iter = 0;
    int x3_iter =0;
    int gain = 0;
    int i;

    BOOL IN_UPCYCLE = FALSE;


        x1.c1 = rand()*(n-1)/RAND_MAX;         //Get first random edge.
        while (1) {
          while ((cost[x1.c1][for_tour[x1.c1]] == inf)   &&   (x1_iter++ < n))
             x1.c1 = mod_int(++x1.c1,n-1);     //Make sure first edge is acceptable.
          if(x1_iter >= (n-1))
		return inf;
          x1.c2 = for_tour[x1.c1];
          while(1){   		                //Get second random edge.
             x2.c1 = rand()*(n-1)/RAND_MAX;
             while (((cost[x2.c1][for_tour[x2.c1]] == inf)||
		     (x1.c1==for_tour[x2.c1])||               
		     (x2.c1==x1.c1))    &&   (x2_iter++<n))
                x2.c1 = mod_int(++x2.c1,n-1);   //Make sure second edge is acceptable.
             if(x2_iter >=n){
                x2_iter = 0;
                break;
             }
             x2.c2 = for_tour[x2.c1];
	     if(TRD_RAND){	//Get third random edge and decide if IN_UPCYCLE.
	        while ((cost[x3.c1=(rand()*(n-1)/RAND_MAX)][for_tour[x3.c1]] == inf)||
		    (x2.c1 == for_tour[x3.c1])||             
		    (x3.c1 == x1.c1)||(x3.c1==x2.c1)); //Make sure third edge is acceptable.
                tmp_edge.c1 = x3.c1;; 
	        tmp_edge.c2 = x3.c1;
                while ((tmp_edge.c1!=x2.c1)&&(tmp_edge.c2!=x1.c2)){ 
	           if((tmp_edge.c1==x1.c1)||(tmp_edge.c2==x2.c2)){
		      IN_UPCYCLE = TRUE;
		      break;
                   }
	        tmp_edge.c1 = for_tour[tmp_edge.c1];
	        tmp_edge.c2 = rev_tour[tmp_edge.c2];
	        }
	     }
	     else{
	        IN_UPCYCLE = TRUE;
		if((x3.c1 = for_tour[x2.c2])==x1.c1){
		   continue;

		}
	     }
             x3.c2 = for_tour[x3.c1];
	     if(IN_UPCYCLE){		//IN_UPCYCLE is true if third edge is between		
	        y1.c1 = x1.c1;		//the second edge and the first edge (upper cycle)
                y1.c2 = x2.c2;		//else the third edge is between the first edge
                y2.c1 = x2.c1;		//and the second edge (lower cycle).
                y2.c2 = x3.c2;
                y3.c1 = x3.c1;
                y3.c2 = x1.c2;
	     }
             else {
	        y1.c1 = x2.c1;
                y1.c2 = x1.c2;
                y2.c1 = x1.c1;	
                y2.c2 = x3.c2;
                y3.c1 = x3.c1;
                y3.c2 = x2.c2;
  	     }
             gain = 0;
	     gain += cost[x1.c1][x1.c2];
	     gain += cost[x2.c1][x2.c2];
	     gain += cost[x3.c1][x3.c2];
	     gain -= cost[y1.c1][y1.c2];
	     gain -= cost[y2.c1][y2.c2];
	     gain -= cost[y3.c1][y3.c2];
             for_tour[y1.c1] = y1.c2;     /*for clarity*/
	     for_tour[y2.c1] = y2.c2;
	     for_tour[y3.c1] = y3.c2;
	     return gain;
          }
      }
}
    


int main(int argc, char *argv[])
{
	int total_MST;
	int * tour;                 /* The tour */
	int * for_tour_copy;        /* Copy of tour - for forward traversal purposes */
	int * rev_tour_copy;        /* Copy of tour - for backward traversal purposes */
	int gain;                   /* Change in energy - positive when a better tour */
	int dummy;                  /*                    found ... else negative    */
	int i,j,n,old,newnum,first_node;
        long double tempx,tempy;
        BOOL do_again = FALSE;      /* FLAG which determines whether to compute other matrix */  		


	while(1) {
	   /* Get all of the paramaters from the user */
        
	   Entered = FALSE;
	   while(((strcmp("2",which_opt) != 0) && (strcmp("3",which_opt) != 0)) || (Entered == FALSE)){
		printf("Specify whether to run 3opt or 2+1opt (3/2 or x=exit)? ");
		scanf("%s", which_opt);
		printf("\n");
		Entered = TRUE;
		if (strcmp("x",which_opt) == 0)
			return 0;
		if ((strcmp("2",which_opt) != 0) && (strcmp("3",which_opt) != 0)){
			printf("You entred:  %s\n", which_opt);
			printf("Please enter 2 (2+1opt) or 3 (3opt).\n");
		}
	   }
	   if (strcmp("2",which_opt) == 0)
		TRD_RAND = FALSE;
	   else
		TRD_RAND = TRUE;
	   Entered = FALSE;
	   if ((!do_again)||((do_again)&&(which_matrix == "f"))){	
	      while(((strcmp("f",which_matrix) != 0) && (strcmp("r",which_matrix) != 0)) || (Entered == FALSE)){
		printf("Specify a file name or generate a random matrix (f/r or x=exit)? ");
		scanf("%s", which_matrix);
		printf("\n");
		Entered = TRUE;
		if (strcmp("x",which_matrix) == 0)
			return 0;
		if ((strcmp("f",which_matrix) != 0) && (strcmp("r",which_matrix) != 0)){
			printf("You entred:  %s\n", which_matrix);
			printf("Please enter f (file) or r (random).\n");
		}
	      }
	      if(strcmp("f",which_matrix) == 0){
		printf("What is the name of the file to process (x=exit)? ");
		scanf("%s", file_name);
		printf("\n");
	      }
	      if(strcmp("r",which_matrix) == 0){
		printf("Please enter a temporary name for the random matrix (x=exit)? ");
		scanf("%s", file_name);
		printf("\n");
	      }
	      if(strcmp("x",file_name) == 0){
		printf("\n");
		break;
	      }     
	      if(strcmp("f",which_matrix) == 0){
	         if((fp = fopen(file_name, "r")) == NULL) {
  		   printf("tsp: cannot open %s\n", file_name);
  		   exit(1);
	         }
	         fscanf(fp,"%d%d",&n, &inf);
     		      }	
	      else{
		printf("Please enter n: ");
		scanf("%d", &n);
		printf("\n");
		inf = 99999;
	      }
	   }
	   /* Initialize all dynamic arrays */
	   parent = (int*)  new int [n*sizeof(int)];
	   tour  = (int*)  new int [n*sizeof(int)];
	   for_tour_copy  = (int*)  new int [n*sizeof(int)];
	   rev_tour_copy  = (int*)  new int [n*sizeof(int)];
	   if((!do_again)||((do_again)&&(which_matrix == "f"))){	
	      matrix = (int**) new int [n*sizeof(int*)];
	      for (i=0; i<n; i++) {
		matrix[i] = (int *) new int [n*sizeof(int)];
		tour[i] = -1;
		for (j=0; j<n; j++)
		   if(strcmp("f",which_matrix) == 0)
	   		fscanf(fp, "%d", &matrix[i][j]);  /* Read matrix from file */
		   else				     /* or generate random    */
			if(i!=j)		     /* matrix                */
			   matrix[i][j] = rand()*400/RAND_MAX; 
			else
		           matrix[i][j] = 0;
	        }
           }
	   else 
	      for (i=0; i<n; i++) 
		tour[i] = -1;
	   total_MST=0;
	   *length = 0;
	   Entered = FALSE;
	   /* More user input processing */
	   while(((strcmp("y",display_matrix) != 0) && (strcmp("n",display_matrix) != 0)) || (Entered == FALSE)){
	      printf("Would you like the input data displayed (y/n)? ");
	      scanf("%s", display_matrix);
	      printf("\n");
	      Entered = TRUE;
	      if ((strcmp("y",display_matrix) != 0) && (strcmp("n",display_matrix) != 0)){
		printf("You entred:  %s\n", display_matrix);
		printf("Please enter y (yes) or n (no).\n");
	      }
	  }
	  if (strcmp("y",display_matrix) == 0){
	      printf("n = %d\n", n);
	      printf("infinity = %d\n\n", inf);		
	      printf("Matrix:");
      	      for (i=0; i<n; i++) {
	         printf("\n");
	   	 for (j=0; j<n; j++) 
	      	    printf("[%5d] ", matrix[i][j]);
       	      }
	  }
	  printf("\n");
	  Entered = FALSE;
	  while(((strcmp("y",process_MST) != 0) && (strcmp("n",process_MST) != 0)) || (Entered == FALSE)){
	    printf("Would you like to process the MST (y/n)? ");
	    scanf("%s", process_MST);
	    printf("\n");
	    Entered = TRUE;
	    if ((strcmp("y",process_MST) != 0) && (strcmp("n",process_MST) != 0)){
	       printf("You entred:  %s\n", process_MST);
	       printf("Please enter y (yes) or n (no).\n");
	    }
	  }
	  if (strcmp("n",process_MST)!=0)
	    if(MST_prim(matrix, parent, n)){
	       Entered = FALSE;
	       while(((strcmp("d",detailedMST) != 0) && (strcmp("b",detailedMST) != 0)) || (Entered == FALSE)){
	          printf("Would you like detailed or brief MST info (d/b)? ");
		  scanf("%s", detailedMST);
		  printf("\n");
		  Entered = TRUE;
		  if ((strcmp("d",detailedMST) != 0) && (strcmp("b",detailedMST) != 0)){
		     printf("You entred:  %s\n", detailedMST);
		     printf("Please enter d (detailed) or b (brief).\n");
	          }
	        }
		if (strcmp("d",detailedMST) == 0)
		   printf("Parent List (Parent -> Child [Weight]):\n");
			/* Display the MST */
		   for(i=0;i<n;i++)
	       	      if (parent[i]>-1){
		         total_MST = total_MST + matrix[parent[i]][i];
			 if (strcmp("d",detailedMST) == 0){
		            printf("%d ",parent[i]);
	                    printf(" -> %d ",i);
			 }
	                 if (matrix[parent[i]][i]<matrix[i][parent[i]]){
			    if (strcmp("d",detailedMST) == 0)
	                       printf("[%d]\n",matrix[parent[i]][i]);
			       total_MST = total_MST + matrix[parent[i]][i];
			 }
	                 else if (strcmp("d",detailedMST) == 0)
	                    printf("[%d]\n",matrix[i][parent[i]]);
		      } 
		      printf("\n");
		      printf("Total MST weight = %d\n", total_MST);
	    }
	    else 
	       printf("MST does not exist!");
	    printf("\n");	
	    /* TSP display part */
            if(init_tour(matrix,tour,0,n,0,0,length)){  //Returns initial "tour" is False otherwise
	       printf("Tour exists!\n\n");
	       Entered = FALSE;
	       while(((strcmp("d",detailedTSP) != 0) && (strcmp("b",detailedTSP) != 0)) || (Entered == FALSE)){
		  printf("Would you like detailed or brief TSP info (d/b)? ");
	 	  scanf("%s", detailedTSP);
	 	  printf("\n");
		  Entered = TRUE;
		  if ((strcmp("d",detailedTSP) != 0) && (strcmp("b",detailedTSP) != 0)){
	             printf("You entred:  %s\n", detailedTSP);
		     printf("Please enter d (detailed) or b (brief).\n");
		  }
	       }
	       printf("Initial tour length = %d\n\n",*length);
               printf("Tour = ");
	       first_node = 0;
	       for(i=0;i<n;i++){
	          printf(" %d",tour[first_node]);
		  first_node = tour[first_node];
	       }
	       printf(" %d\n\n",tour[0]);
		/* Display the TSP route */
	       if (strcmp("d",detailedTSP) == 0){
                  printf("Edge weights of tour:\n");
		  for(i=0;i<(n-1);i++)
		     printf("%d -> %d [%d] \n",tour[i], tour[i+1], matrix[tour[i]][tour[i+1]]);
                     printf("%d -> %d [%d] \n",tour[i+1], tour[0], matrix[tour[i+1]][tour[0]]);
		     printf("\n");
		  }
	       printf("\n");  			//Prompts for the Annealing parameters T and a.
	       printf("Enter initial temperature T (enter 0 to quit): ");
 	       scanf("%lf",&INIT_TEMP);
 	       printf("\n");
 	       printf("Enter schedule parameter 'a' for {T = aT | 0 < a <1}: "); 
 	       scanf("%lf",&ALPHA);
 	       printf("\n");			
	       copytour(tour,for_tour_copy,rev_tour_copy, n);	
	       TEMP = INIT_TEMP;
	       start = clock()/CLOCKS_PER_SEC;			
	       while (TEMP > ZERO) {		//Simulated Annealing iterative loop.
                  gain = do3opt(matrix,for_tour_copy,rev_tour_copy,n);
     	   	  if (gain > 0){
		     *length -= gain;
       	      	     copytour(for_tour_copy, tour,rev_tour_copy, n);
		  }
     	   	  else {
                     tempx = ((long double) rand())/((long double)RAND_MAX);
		     tempy = exp(((long double)gain)/TEMP);
       	      	     if(tempx <= tempy){
			*length -= gain;
                        copytour(for_tour_copy, tour,rev_tour_copy, n);
		     }
       	             else
	         	copytour(tour, for_tour_copy, rev_tour_copy, n);
		  }
    	   	  TEMP *= ALPHA;
	       }
	       finish = clock()/CLOCKS_PER_SEC;
	       result(length,n);
	       free (parent);
	       free (tour); 
	       free (for_tour_copy);
	       free (rev_tour_copy);
	       do_again = FALSE;
	   }
        else
		printf("Tour does not exist\n");
	Entered = FALSE;    	
	while(((strcmp("r",no_new) != 0) && (strcmp("n",no_new) != 0)) || (Entered == FALSE)){
		printf("Repeat with same values or get new data (r/n)? ");
		scanf("%s", no_new);
		printf("\n");			
		Entered = TRUE;
		if (strcmp("x",no_new) == 0)
		   return 0;
		if ((strcmp("r",no_new) != 0) && (strcmp("n",no_new) != 0)){
		   printf("You entred:  %s\n", no_new);
		   printf("Please enter r (repeat) or n (new).\n");
		}
	}
	if (strcmp("r",no_new) == 0)
		do_again = TRUE;
	else{
		for (i=0;i<n;i++)
		  free (matrix[i]);
		free(matrix);	
		do_again = FALSE;
	}    	
}
}
